/*  g_ipc.c - main file
 *  
 *  (C) Copyright XII 2004 Michal 'GiM' Spadlinski <gim at skrzynka dot pl>
 *
 *  This is small IPC test suite.
 *  It tests message queues and shared memory in POSIX and SystemV.
 *  Tests aren't quite good, couse they DON'T use semaphores
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <sys/types.h>
#include <sys/stat.h>
#define __USE_SVID
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>

#include "dynstuff.h"
#include "g_sysv_queue.h"
#include "g_sysv_shared.h"
#include "g_sysv_shared2.h"
#include "g_posix_queue.h"
#include "g_posix_queue2.h"
#include "g_posix_shared.h"
#include "g_ipc_wspolne.h"
#include "g_ipc_opcje.h"

const int long_s = sizeof(long);
const key_t G_KEY=0xdefaced;
int MAX_QUERIES = 10;

const int g_data_format_s = sizeof(struct g_data_format);
struct g_data_format parentp[2], childp[2];
/*
#define TEST_SYSV_QUEUE		0x01
#define TEST_SYSV_SHARED	0x02
#define TEST_SYSV_SHARED2	0x04
#define TEST_POSIX_QUEUE	0x08
#define TEST_POSIX_QUEUE2	0x10
#define TEST_POSIX_SHARED	0x20
*/

int default_prefork(void) { return 0; }
struct test_handler {
	int (*prefork_handler)(void);
	int (*child_handler)(int);
	int (*parent_handler)(int);
} my_test_handlers[] =
{
	{ default_prefork, child_sysv_queue, parent_sysv_queue },
	{ prefork_sysv_shared, child_sysv_shared, parent_sysv_shared },
	{ prefork_sysv_shared2, child_sysv_shared2, parent_sysv_shared2 },
#ifdef HAS_POSIX_QUEUES
	{ prefork_posix_queue, child_posix_queue, parent_posix_queue },
	{ prefork_posix_queue2, child_posix_queue2, parent_posix_queue2 },
#endif
	{ prefork_posix_shared, child_posix_shared, parent_posix_shared }
};

char *memp = NULL;

int showtimes(char *str, list_t p)
{
	int all = 0;
	double min=300, max=0, whole=0, tim;
	struct timeval *tmp;

	while (p)
	{
		tmp = (struct timeval *)(p->data);
		tim = tmp->tv_sec + (tmp->tv_usec/1000000.0);
		DEB("%lu %lu %8.6f\n", tmp->tv_sec, tmp->tv_usec, tim);
		whole += tim;
		if (tim < min) min = tim;
		if (tim > max) max = tim;
		p=p->next; all++;
	}
	printf ("%s: min: %8.6f s avg: %8.6f s max: %8.6f s\n", str, min, (float)(whole/all), max);
	return 0;
}

int main(int argc, char **argv)
{
	int i, pid;
	int mode = 0x0;
	char *namep=*argv;

	if (g_ipc_skanuj_opcje(&argc, &argv))
		return 1;

	g_pokaz_przekazane_opcje();
	for (i=0; i<G_ALL_OPTS; i++)
		if (g_opcje[i].val)
			mode |= (1<<i);

	if (g_opcje[G_ALL_OPTS].val)
		mode |= (1<<G_ALL_OPTS)-1;

	if (g_opcje[G_TESTS_NO].val)
		MAX_QUERIES = g_opcje[G_TESTS_NO].val;

	if (!mode || MAX_QUERIES<0)
		return g_pokaz_pomoc(namep);

	for (i=0; i<sizeof(my_test_handlers)/sizeof(struct test_handler); i++)
	{
		if (mode & (1<<i))
		{
			if (!my_test_handlers[i].prefork_handler())
			{
				pid = fork();
				if (!pid) {
					my_test_handlers[i].child_handler(0);
					return (0);
				} else {
					if (pid != -1) {
						my_test_handlers[i].parent_handler(pid);
					} else
						perror("fork failed");
				}
			}
		}
	}

	return 0;
}

#ifndef DEBUG
int DEB(char *x, ...)
{
	return 0;
}
#endif
